@@ -393,24 +393,34 @@ static void scroll_callback(GLFWwindow* window, double x, double y)
 static void key_callback(GLFWwindow* window, int key, int scancode, int action, int mods)
 {
     Slot* slot = glfwGetWindowUserPointer(window);
-    const char* name = glfwGetKeyName(key, scancode);
 
-    if (name)
+    if (key == GLFW_KEY_UNKNOWN)
     {
-        printf("%08x to %i at %0.3f: Key 0x%04x Scancode 0x%04x (%s) (%s) (with%s) was %s\n",
-               counter++, slot->number, glfwGetTime(), key, scancode,
-               get_key_name(key),
-               name,
-               get_mods_name(mods),
-               get_action_name(action));
+        printf("%08x to %i at %0.3f: Key (%s) Scancode 0x%04x (with%s) was %s\n",
+                counter++, slot->number, glfwGetTime(),
+                get_key_name(key), scancode,
+                get_mods_name(mods),
+                get_action_name(action));
     }
     else
     {
-        printf("%08x to %i at %0.3f: Key 0x%04x Scancode 0x%04x (%s) (with%s) was %s\n",
-               counter++, slot->number, glfwGetTime(), key, scancode,
-               get_key_name(key),
-               get_mods_name(mods),
-               get_action_name(action));
+        const char* name = glfwGetKeyName(key, scancode);
+        if (name)
+        {
+            printf("%08x to %i at %0.3f: Key 0x%04x (%s) Scancode 0x%04x Name %s (with%s) was %s\n",
+                   counter++, slot->number, glfwGetTime(),
+                   key, get_key_name(key), scancode, name,
+                   get_mods_name(mods),
+                   get_action_name(action));
+        }
+        else
+        {
+            printf("%08x to %i at %0.3f: Key 0x%04x (%s) Scancode 0x%04x (with%s) was %s\n",
+                   counter++, slot->number, glfwGetTime(),
+                   key, get_key_name(key), scancode,
+                   get_mods_name(mods),
+                   get_action_name(action));
+        }
     }
 
     if (action != GLFW_PRESS)
